@@ -54,8 +54,19 @@ def build(v: str):
     if os.path.exists("dist"):
         shutil.rmtree("dist")
     os.mkdir("dist")
-    subprocess.check_call(f"git checkout releases-{v}", shell=True)
     branch = f"releases-{v}"
+    # Check if branch exists, if not create it
+    result = subprocess.run(
+        f"git show-ref --verify --quiet refs/heads/{branch}",
+        shell=True,
+        capture_output=True,
+    )
+    if result.returncode == 0:
+        # Branch exists, checkout
+        subprocess.check_call(f"git checkout {branch}", shell=True)
+    else:
+        # Branch doesn't exist, create it
+        subprocess.check_call(f"git checkout -b {branch}", shell=True)
     src_tar = f"apache-fory-{v}-src.tar.gz"
     _check_all_committed()
     _strip_unnecessary_license()
@@ -178,14 +189,22 @@ def bump_java_version(new_version):
         "integration_tests/latest_jdk_tests",
         "integration_tests/latest_jdk_tests",
         "java/benchmark",
+        "java/fory-core",
+        "java/fory-format",
+        "java/fory-simd",
+        "java/fory-extensions",
+        "java/fory-test-core",
+        "java/fory-testsuite",
     ]:
         _bump_version(p, "pom.xml", new_version, _update_pom_parent_version)
-    os.chdir(os.path.join(PROJECT_ROOT_DIR, "java"))
-    subprocess.check_output(
-        f"mvn versions:set -DnewVersion={new_version}",
-        shell=True,
-        universal_newlines=True,
-    )
+    # mvn versions:set too slow
+    # os.chdir(os.path.join(PROJECT_ROOT_DIR, "java"))
+    # subprocess.check_output(
+    #     f"mvn versions:set -DnewVersion={new_version}",
+    #     shell=True,
+    #     universal_newlines=True,
+    # )
+    _bump_version("java", "pom.xml", new_version, _update_parent_pom_version)
 
 
 def _update_pom_parent_version(lines, new_version):
@@ -216,9 +235,17 @@ def _update_scala_version(lines, v):
 
 
 def _update_kotlin_version(lines, v):
+    return _update_pom_version(lines, v, "<artifactId>fory-kotlin</artifactId>")
+
+
+def _update_parent_pom_version(lines, v):
+    return _update_pom_version(lines, v, "<packaging>pom</packaging>")
+
+
+def _update_pom_version(lines, v, prev):
     target_index = -1
     for index, line in enumerate(lines):
-        if "<artifactId>fory-kotlin</artifactId>" in line:
+        if prev in line:
             target_index = index + 1
             break
     current_version_line = lines[target_index]
