@@ -372,11 +372,11 @@ case $1 in
     ;;
     go)
       echo "Executing fory go tests for go"
-      # cd "$ROOT/go/fory"
-      # go install ./cmd/fory
-      # cd "$ROOT/go/fory/tests"
-      # go generate
-      # go test -v
+      cd "$ROOT/go/fory"
+      go install ./cmd/fory
+      cd "$ROOT/go/fory/tests"
+      go generate
+      go test -v
       cd "$ROOT/go/fory"
       go test -v
       echo "Executing fory go tests succeeds"
