@@ -19,10 +19,11 @@ package fory
 
 import (
 	"fmt"
-	"github.com/stretchr/testify/require"
 	"reflect"
 	"testing"
 	"unsafe"
+
+	"github.com/stretchr/testify/require"
 )
 
 func primitiveData() []interface{} {
@@ -221,7 +222,7 @@ func TestSerializeStructSimple(t *testing.T) {
 		type A struct {
 			F1 []string
 		}
-		require.Nil(t, fory.RegisterTagType("example.A", A{}))
+		require.Nil(t, fory.RegisterNamedType(A{}, "example.A"))
 		serde(t, fory, A{})
 		serde(t, fory, &A{})
 		serde(t, fory, A{F1: []string{"str1", "", "str2"}})
@@ -231,7 +232,7 @@ func TestSerializeStructSimple(t *testing.T) {
 			F1 []string
 			F2 map[string]int32
 		}
-		require.Nil(t, fory.RegisterTagType("example.B", B{}))
+		require.Nil(t, fory.RegisterNamedType(B{}, "example.B"))
 		serde(t, fory, B{})
 		serde(t, fory, B{
 			F1: []string{"str1", "", "str2"},
@@ -243,6 +244,15 @@ func TestSerializeStructSimple(t *testing.T) {
 	}
 }
 
+func TestRegisterById(t *testing.T) {
+	fory := NewFory(false)
+	type simple struct {
+		Field string
+	}
+	require.NoError(t, fory.Register(simple{}, 100))
+	serde(t, fory, simple{Field: "value"})
+}
+
 func TestSerializeBeginWithMagicNumber(t *testing.T) {
 	strSlice := []string{"str1", "str1", "", "", "str2"}
 	fory := NewFory(true)
@@ -291,7 +301,7 @@ func newFoo() Foo {
 func TestSerializeStruct(t *testing.T) {
 	for _, referenceTracking := range []bool{false, true} {
 		fory := NewFory(referenceTracking)
-		require.Nil(t, fory.RegisterTagType("example.Bar", Bar{}))
+		require.Nil(t, fory.RegisterNamedType(Bar{}, "example.Bar"))
 		serde(t, fory, &Bar{})
 		bar := Bar{F1: 1, F2: "str"}
 		serde(t, fory, bar)
@@ -301,54 +311,26 @@ func TestSerializeStruct(t *testing.T) {
 			F1 Bar
 			F2 interface{}
 		}
-		require.Nil(t, fory.RegisterTagType("example.A", A{}))
+		require.Nil(t, fory.RegisterNamedType(A{}, "example.A"))
 		serde(t, fory, A{})
 		serde(t, fory, &A{})
 		serde(t, fory, A{F1: Bar{F1: 1, F2: "str"}, F2: -1})
 		serde(t, fory, &A{F1: Bar{F1: 1, F2: "str"}, F2: -1})
 
-		require.Nil(t, fory.RegisterTagType("example.Foo", Foo{}))
+		require.Nil(t, fory.RegisterNamedType(Foo{}, "example.Foo"))
 		foo := newFoo()
 		serde(t, fory, foo)
 		serde(t, fory, &foo)
 	}
 }
 
-func TestSerializeStringReference(t *testing.T) {
-	fory := NewFory(true)
-	strSlice := []string{"str1", "str1", "", "", "str2"}
-	strSlice = append(strSlice, strSlice[0])
-	serde(t, fory, strSlice)
-	type A struct {
-		F1 string
-		F2 string
-	}
-	require.Nil(t, fory.RegisterTagType("example.A", A{}))
-	serde(t, fory, A{})
-	serde(t, fory, A{F1: "str", F2: "str"})
-	var strData []byte
-	for i := 0; i < 1000; i++ {
-		strData = append(strData, 100)
-	}
-	x := string(strData)
-	serde(t, fory, &x)
-	strSlice2 := []string{x, x, x}
-	bytes, err := fory.Marshal(strSlice2)
-	require.Nil(t, err)
-	require.Less(t, len(bytes), 2*len(strData))
-	strSlice23 := []*string{&x, &x, &x}
-	bytes, err = fory.Marshal(strSlice23)
-	require.Nil(t, err)
-	require.Less(t, len(bytes), 2*len(strData))
-}
-
 func TestSerializeCircularReference(t *testing.T) {
 	fory := NewFory(true)
 	{
 		type A struct {
 			A1 *A
 		}
-		require.Nil(t, fory.RegisterTagType("example.A", A{}))
+		require.Nil(t, fory.RegisterNamedType(A{}, "example.A"))
 		// If use `A{}` instead of `&A{}` and pass `a` instead of `&a`, there will be serialization data duplication
 		// and can't be deserialized by other languages too.
 		// TODO(chaokunyang) If pass by value(have a copy) and there are some inner value reference, return a readable
@@ -368,7 +350,7 @@ func TestSerializeCircularReference(t *testing.T) {
 			F2 *B
 			F3 *B
 		}
-		require.Nil(t, fory.RegisterTagType("example.B", B{}))
+		require.Nil(t, fory.RegisterNamedType(B{}, "example.B"))
 		b := &B{F1: "str"}
 		b.F2 = b
 		b.F3 = b
@@ -396,8 +378,8 @@ func TestSerializeComplexReference(t *testing.T) {
 		F3 *A
 		F4 *B
 	}
-	require.Nil(t, fory.RegisterTagType("example.A", A{}))
-	require.Nil(t, fory.RegisterTagType("example.B", B{}))
+	require.Nil(t, fory.RegisterNamedType(A{}, "example.A"))
+	require.Nil(t, fory.RegisterNamedType(B{}, "example.B"))
 
 	a := &A{F1: "str"}
 	a.F2 = a
@@ -512,8 +494,8 @@ func serde(t *testing.T, fory *Fory, value interface{}) {
 
 func BenchmarkMarshal(b *testing.B) {
 	fory := NewFory(true)
-	require.Nil(b, fory.RegisterTagType("example.Foo", Foo{}))
-	require.Nil(b, fory.RegisterTagType("example.Bar", Bar{}))
+	require.Nil(b, fory.RegisterNamedType(Foo{}, "example.Foo"))
+	require.Nil(b, fory.RegisterNamedType(Bar{}, "example.Bar"))
 	value := benchData()
 	for i := 0; i < b.N; i++ {
 		_, err := fory.Marshal(value)
@@ -525,8 +507,8 @@ func BenchmarkMarshal(b *testing.B) {
 
 func BenchmarkUnmarshal(b *testing.B) {
 	fory := NewFory(true)
-	require.Nil(b, fory.RegisterTagType("example.Foo", Foo{}))
-	require.Nil(b, fory.RegisterTagType("example.Bar", Bar{}))
+	require.Nil(b, fory.RegisterNamedType(Foo{}, "example.Foo"))
+	require.Nil(b, fory.RegisterNamedType(Bar{}, "example.Bar"))
 	value := benchData()
 	data, err := fory.Marshal(value)
 	if err != nil {
@@ -638,10 +620,10 @@ func TestStructWithNestedSlice(t *testing.T) {
 	}
 
 	fory := NewFory(true)
-	if err := fory.RegisterTagType("Example", Example{}); err != nil {
+	if err := fory.RegisterNamedType(Example{}, "Example"); err != nil {
 		panic(err)
 	}
-	if err := fory.RegisterTagType("Item", Item{}); err != nil {
+	if err := fory.RegisterNamedType(Item{}, "Item"); err != nil {
 		panic(err)
 	}
 
