@@ -19,13 +19,15 @@ package fory
 
 import (
 	"fmt"
-	"github.com/apache/fory/go/fory/meta"
 	"hash/fnv"
 	"reflect"
 	"regexp"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
+
+	"github.com/apache/fory/go/fory/meta"
 )
 
 type TypeId = int16
@@ -69,8 +71,8 @@ const (
 	NAMED_STRUCT = 17
 	// NAMED_COMPATIBLE_STRUCT a compatible_struct whose type mapping will be encoded as a name
 	NAMED_COMPATIBLE_STRUCT = 18
-	// EXTENSION a type which will be serialized by a customized serializer
-	EXTENSION = 19
+	// EXT a type which will be serialized by a customized serializer
+	EXT = 19
 	// NAMED_EXT an ext type whose type mapping will be encoded as a name
 	NAMED_EXT = 20
 	// LIST A list of some logical data type
@@ -111,6 +113,8 @@ const (
 	ARROW_RECORD_BATCH = 38
 	// ARROW_TABLE an arrow table object
 	ARROW_TABLE = 39
+	// UNKNOWN an unknown type
+	UNKNOWN = 63
 
 	// UINT8 Unsigned 8-bit little-endian integer
 	UINT8 = 100 // Not in mapping table, assign a higher value
@@ -182,6 +186,7 @@ var namedTypes = map[TypeId]struct{}{
 
 // IsNamespacedType checks whether the given type ID is a namespace type
 func IsNamespacedType(typeID TypeId) bool {
+	typeID = typeID & 0xFF
 	_, exists := namedTypes[typeID]
 	return exists
 }
@@ -202,6 +207,7 @@ var (
 	stringSliceType    = reflect.TypeOf((*[]string)(nil)).Elem()
 	byteSliceType      = reflect.TypeOf((*[]byte)(nil)).Elem()
 	boolSliceType      = reflect.TypeOf((*[]bool)(nil)).Elem()
+	int8SliceType      = reflect.TypeOf((*[]int8)(nil)).Elem()
 	int16SliceType     = reflect.TypeOf((*[]int16)(nil)).Elem()
 	int32SliceType     = reflect.TypeOf((*[]int32)(nil)).Elem()
 	int64SliceType     = reflect.TypeOf((*[]int64)(nil)).Elem()
@@ -223,6 +229,26 @@ var (
 	genericSetType     = reflect.TypeOf((*GenericSet)(nil)).Elem()
 )
 
+// Global registry for generated serializer factories
+var generatedSerializerFactories = struct {
+	mu        sync.RWMutex
+	factories map[reflect.Type]func() Serializer
+}{
+	factories: make(map[reflect.Type]func() Serializer),
+}
+
+// RegisterSerializerFactory registers a factory function for a generated serializer
+func RegisterSerializerFactory(type_ interface{}, factory func() Serializer) {
+	reflectType := reflect.TypeOf(type_)
+	if reflectType.Kind() == reflect.Ptr {
+		reflectType = reflectType.Elem()
+	}
+
+	generatedSerializerFactories.mu.Lock()
+	defer generatedSerializerFactories.mu.Unlock()
+	generatedSerializerFactories.factories[reflectType] = factory
+}
+
 type TypeInfo struct {
 	Type          reflect.Type
 	FullNameBytes []byte
@@ -283,6 +309,10 @@ type typeResolver struct {
 	namespaceDecoder *meta.Decoder
 	typeNameEncoder  *meta.Encoder
 	typeNameDecoder  *meta.Decoder
+
+	// meta share related
+	typeToTypeDef  map[reflect.Type]*TypeDef
+	defIdToTypeDef map[int64]*TypeDef
 }
 
 func newTypeResolver(fory *Fory) *typeResolver {
@@ -318,6 +348,9 @@ func newTypeResolver(fory *Fory) *typeResolver {
 		namespaceDecoder: meta.NewDecoder('.', '_'),
 		typeNameEncoder:  meta.NewEncoder('$', '_'),
 		typeNameDecoder:  meta.NewDecoder('$', '_'),
+
+		typeToTypeDef:  make(map[reflect.Type]*TypeDef),
+		defIdToTypeDef: make(map[int64]*TypeDef),
 	}
 	// base type info for encode/decode types.
 	// composite types info will be constructed dynamically.
@@ -341,6 +374,55 @@ func newTypeResolver(fory *Fory) *typeResolver {
 		r.typeToTypeInfo[t] = t.String()
 	}
 	r.initialize()
+
+	// Register generated serializers from factories with complete type information
+	generatedSerializerFactories.mu.RLock()
+	for type_, factory := range generatedSerializerFactories.factories {
+		codegenSerializer := factory()
+		pkgPath := type_.PkgPath()
+		typeName := type_.Name()
+		typeTag := pkgPath + "." + typeName
+
+		// Create structSerializer with codegen delegate (Python-style)
+		structSer := &structSerializer{
+			typeTag:         typeTag,
+			type_:           type_,
+			codegenDelegate: codegenSerializer, // Delegate to codegen for performance
+		}
+		// Create ptrToStructSerializer with codegen delegate (Python-style)
+		ptrType := reflect.PtrTo(type_)
+		ptrStructSer := &ptrToStructSerializer{
+			type_:            ptrType,
+			structSerializer: *structSer,
+			codegenDelegate:  codegenSerializer, // Delegate to codegen for performance
+		}
+
+		// 1. Basic type mappings (same as reflection)
+		r.typeToSerializers[type_] = structSer      // Value type -> structSerializer with codegen
+		r.typeToSerializers[ptrType] = ptrStructSer // Pointer type -> ptrToStructSerializer with codegen
+
+		// 2. Cross-language critical mapping (same as reflection)
+		r.typeTagToSerializers[typeTag] = ptrStructSer // "pkg.Type" -> ptrToStructSerializer
+
+		// 3. Register complete type information (critical for proper serialization)
+		_, err := r.registerType(type_, int32(codegenSerializer.TypeId()), pkgPath, typeName, structSer, false)
+		if err != nil {
+			panic(fmt.Errorf("failed to register codegen type %s: %v", typeTag, err))
+		}
+		// 4. Register pointer type information
+		_, err = r.registerType(ptrType, -int32(codegenSerializer.TypeId()), pkgPath, typeName, ptrStructSer, false)
+		if err != nil {
+			panic(fmt.Errorf("failed to register codegen pointer type %s: %v", typeTag, err))
+		}
+
+		// 5. Type info mappings (same as reflection) - these are redundant if registerType is called
+		r.typeToTypeInfo[type_] = "@" + typeTag    // Type -> "@pkg.Type"
+		r.typeToTypeInfo[ptrType] = "*@" + typeTag // *Type -> "*@pkg.Type"
+		r.typeInfoToType["@"+typeTag] = type_      // "@pkg.Type" -> Type
+		r.typeInfoToType["*@"+typeTag] = ptrType   // "*@pkg.Type" -> *Type
+	}
+	generatedSerializerFactories.mu.RUnlock()
+
 	return r
 }
 
@@ -353,12 +435,13 @@ func (r *typeResolver) initialize() {
 		{stringPtrType, ptrToStringSerializer{}},
 		{stringSliceType, stringSliceSerializer{}},
 		{byteSliceType, byteSliceSerializer{}},
-		{boolSliceType, boolSliceSerializer{}},
-		{int16SliceType, int16SliceSerializer{}},
-		{int32SliceType, int32SliceSerializer{}},
-		{int64SliceType, int64SliceSerializer{}},
-		{float32SliceType, float32SliceSerializer{}},
-		{float64SliceType, float64SliceSerializer{}},
+		// Map basic type slices to proper array types for xlang compatibility
+		{boolSliceType, boolArraySerializer{}},
+		{int16SliceType, int16ArraySerializer{}},
+		{int32SliceType, int32ArraySerializer{}},
+		{int64SliceType, int64ArraySerializer{}},
+		{float32SliceType, float32ArraySerializer{}},
+		{float64SliceType, float64ArraySerializer{}},
 		{interfaceSliceType, sliceSerializer{}},
 		{interfaceMapType, mapSerializer{}},
 		{boolType, boolSerializer{}},
@@ -399,11 +482,34 @@ func (r *typeResolver) RegisterSerializer(type_ reflect.Type, s Serializer) erro
 	return nil
 }
 
-func (r *typeResolver) RegisterTypeTag(value reflect.Value, tag string) error {
-	type_ := value.Type()
+func (r *typeResolver) RegisterNamedType(
+	type_ reflect.Type,
+	typeId int32,
+	namespace string,
+	typeName string,
+) error {
 	if prev, ok := r.typeToSerializers[type_]; ok {
 		return fmt.Errorf("type %s already has a serializer %s registered", type_, prev)
 	}
+	registerById := (typeId != 0)
+	if registerById && typeName != "" {
+		return fmt.Errorf("typename %s and typeId %d cannot be both register", typeName, typeId)
+	}
+	if namespace == "" {
+		if idx := strings.LastIndex(typeName, "."); idx != -1 {
+			namespace = typeName[:idx]
+			typeName = typeName[idx+1:]
+		}
+	}
+	if typeName == "" && namespace != "" {
+		return fmt.Errorf("typeName cannot be empty if namespace is provided")
+	}
+	var tag string
+	if namespace == "" {
+		tag = typeName
+	} else {
+		tag = namespace + "." + typeName
+	}
 	serializer := &structSerializer{type_: type_, typeTag: tag}
 	r.typeToSerializers[type_] = serializer
 	// multiple struct with same name defined inside function will have same `type_.String()`, but they are
@@ -413,21 +519,38 @@ func (r *typeResolver) RegisterTypeTag(value reflect.Value, tag string) error {
 	r.typeInfoToType["@"+tag] = type_
 
 	ptrType := reflect.PtrTo(type_)
-	ptrValue := reflect.New(type_)
 	ptrSerializer := &ptrToStructSerializer{structSerializer: *serializer, type_: ptrType}
 	r.typeToSerializers[ptrType] = ptrSerializer
 	// use `ptrToStructSerializer` as default deserializer when deserializing data from other languages.
 	r.typeTagToSerializers[tag] = ptrSerializer
 	r.typeToTypeInfo[ptrType] = "*@" + tag
 	r.typeInfoToType["*@"+tag] = ptrType
-	// For named structs, directly register both their value and pointer types
-	info, err := r.getTypeInfo(value, true)
-	if err != nil {
-		return fmt.Errorf("failed to register named structs: info is %v", info)
+	if typeId == 0 {
+		if r.metaShareEnabled() {
+			typeId = (typeId << 8) + NAMED_COMPATIBLE_STRUCT
+		} else {
+			typeId = (typeId << 8) + NAMED_STRUCT
+		}
+	} else {
+		if r.metaShareEnabled() {
+			typeId = COMPATIBLE_STRUCT
+		} else {
+			typeId = STRUCT
+		}
 	}
-	info, err = r.getTypeInfo(ptrValue, true)
+	if registerById {
+		if info, ok := r.typeIDToTypeInfo[typeId]; ok {
+			return fmt.Errorf("type %s with id %d has been registered", info.Type, typeId)
+		}
+	}
+	// For named structs, directly register both their value and pointer types
+	_, err := r.registerType(type_, typeId, namespace, typeName, nil, false)
 	if err != nil {
-		return fmt.Errorf("failed to register named structs: info is %v", info)
+		return fmt.Errorf("failed to register named structs: %w", err)
+	}
+	_, err = r.registerType(ptrType, -typeId, namespace, typeName, nil, false)
+	if err != nil {
+		return fmt.Errorf("failed to register named structs: %w", err)
 	}
 	return nil
 }
@@ -460,6 +583,10 @@ func (r *typeResolver) getSerializerByTypeTag(typeTag string) (Serializer, error
 
 func (r *typeResolver) getTypeInfo(value reflect.Value, create bool) (TypeInfo, error) {
 	// First check if type info exists in cache
+	if value.Kind() == reflect.Interface {
+		// make sure the concrete value don't miss its real typeInfo
+		value = value.Elem()
+	}
 	typeString := value.Type()
 	if info, ok := r.typesInfo[typeString]; ok {
 		if info.Serializer == nil {
@@ -483,35 +610,27 @@ func (r *typeResolver) getTypeInfo(value reflect.Value, create bool) (TypeInfo,
 	if !create {
 		fmt.Errorf("type %v not registered and create=false", value.Type())
 	}
-	if value.Kind() == reflect.Interface {
-		value = value.Elem()
-	}
-	typ := value.Type()
+	type_ := value.Type()
 	// Get package path and type name for registration
 	var typeName string
 	var pkgPath string
-	rawInfo, ok := r.typeToTypeInfo[typ]
+	rawInfo, ok := r.typeToTypeInfo[type_]
 	if !ok {
-		fmt.Errorf("type %v not registered with a tag", typ)
+		fmt.Errorf("type %v not registered with a tag", type_)
 	}
 	clean := strings.TrimPrefix(rawInfo, "*@")
 	clean = strings.TrimPrefix(clean, "@")
+	typeName = clean
 	if idx := strings.LastIndex(clean, "."); idx != -1 {
 		pkgPath = clean[:idx]
-	} else {
-		pkgPath = clean
-	}
-	if typ.Kind() == reflect.Ptr {
-		typeName = typ.Elem().Name()
-	} else {
-		typeName = typ.Name()
+		typeName = clean[idx+1:]
 	}
 
 	// Handle special types that require explicit registration
 	switch {
-	case typ.Kind() == reflect.Ptr:
+	case type_.Kind() == reflect.Ptr:
 		fmt.Errorf("pointer types must be registered explicitly")
-	case typ.Kind() == reflect.Interface:
+	case type_.Kind() == reflect.Interface:
 		fmt.Errorf("interface types must be registered explicitly")
 	case pkgPath == "" && typeName == "":
 		fmt.Errorf("anonymous types must be registered explicitly")
@@ -524,7 +643,7 @@ func (r *typeResolver) getTypeInfo(value reflect.Value, create bool) (TypeInfo,
 		// Auto-assign IDs
 		typeID = 0
 	default:
-		fmt.Errorf("type %v must be registered explicitly", typ)
+		fmt.Errorf("type %v must be registered explicitly", type_)
 	}
 
 	/*
@@ -547,8 +666,8 @@ func (r *typeResolver) getTypeInfo(value reflect.Value, create bool) (TypeInfo,
 	} else if value.Kind() == reflect.Map {
 		typeID = MAP
 	} else if value.Kind() == reflect.Array {
-		typ = reflect.SliceOf(typ.Elem())
-		return r.typesInfo[typ], nil
+		type_ = reflect.SliceOf(type_.Elem())
+		return r.typesInfo[type_], nil
 	} else if isMultiDimensionaSlice(value) {
 		typeID = LIST
 		return r.typeIDToTypeInfo[typeID], nil
@@ -556,7 +675,7 @@ func (r *typeResolver) getTypeInfo(value reflect.Value, create bool) (TypeInfo,
 
 	// Register the type with full metadata
 	return r.registerType(
-		typ,
+		type_,
 		typeID,
 		pkgPath,
 		typeName,
@@ -574,7 +693,7 @@ func isMultiDimensionaSlice(v reflect.Value) bool {
 }
 
 func (r *typeResolver) registerType(
-	typ reflect.Type,
+	type_ reflect.Type,
 	typeID int32,
 	namespace string,
 	typeName string,
@@ -582,24 +701,24 @@ func (r *typeResolver) registerType(
 	internal bool,
 ) (TypeInfo, error) {
 	// Input validation
-	if typ == nil {
+	if type_ == nil {
 		panic("nil type")
 	}
 	if typeName == "" && namespace != "" {
 		panic("namespace provided without typeName")
 	}
 	if internal && serializer != nil {
-		if err := r.RegisterSerializer(typ, serializer); err != nil {
+		if err := r.RegisterSerializer(type_, serializer); err != nil {
 			panic(fmt.Errorf("impossible error: %s", err))
 		}
 	}
 	// Serializer initialization
 	if !internal && serializer == nil {
 		var err error
-		serializer = r.typeToSerializers[typ] // Check pre-registered serializers
+		serializer = r.typeToSerializers[type_] // Check pre-registered serializers
 		if serializer == nil {
 			// Create new serializer if not found
-			if serializer, err = r.createSerializer(typ, false); err != nil {
+			if serializer, err = r.createSerializer(type_, false); err != nil {
 				panic(fmt.Sprintf("failed to create serializer: %v", err))
 			}
 		}
@@ -632,16 +751,16 @@ func (r *typeResolver) registerType(
 
 	// Build complete type information structure
 	typeInfo := TypeInfo{
-		Type:         typ,
+		Type:         type_,
 		TypeID:       typeID,
 		Serializer:   serializer,
 		PkgPathBytes: nsBytes,   // Encoded namespace bytes
 		NameBytes:    typeBytes, // Encoded type name bytes
 		IsDynamic:    dynamicType,
-		hashValue:    calcTypeHash(typ), // Precomputed hash for fast lookups
+		hashValue:    calcTypeHash(type_), // Precomputed hash for fast lookups
 	}
 	// Update resolver caches:
-	r.typesInfo[typ] = typeInfo // Cache by type string
+	r.typesInfo[type_] = typeInfo // Cache by type string
 	if typeName != "" {
 		r.namedTypeToTypeInfo[[2]string{namespace, typeName}] = typeInfo
 		// Cache by hashed namespace/name bytes
@@ -671,15 +790,19 @@ func (r *typeResolver) registerType(
 	return typeInfo, nil
 }
 
-func calcTypeHash(typ reflect.Type) uint64 {
+func calcTypeHash(type_ reflect.Type) uint64 {
 	// Implement proper hash calculation based on type
 	h := fnv.New64a()
-	h.Write([]byte(typ.PkgPath()))
-	h.Write([]byte(typ.Name()))
-	h.Write([]byte(typ.Kind().String()))
+	h.Write([]byte(type_.PkgPath()))
+	h.Write([]byte(type_.Name()))
+	h.Write([]byte(type_.Kind().String()))
 	return h.Sum64()
 }
 
+func (r *typeResolver) metaShareEnabled() bool {
+	return r.fory != nil && r.fory.metaContext != nil && r.fory.compatible
+}
+
 func (r *typeResolver) writeTypeInfo(buffer *ByteBuffer, typeInfo TypeInfo) error {
 	// Extract the internal type ID (lower 8 bits)
 	typeID := typeInfo.TypeID
@@ -693,6 +816,12 @@ func (r *typeResolver) writeTypeInfo(buffer *ByteBuffer, typeInfo TypeInfo) erro
 
 	// For namespaced types, write additional metadata:
 	if IsNamespacedType(TypeId(internalTypeID)) {
+		if r.metaShareEnabled() {
+			if err := r.writeSharedTypeMeta(buffer, typeInfo); err != nil {
+				return err
+			}
+			return nil
+		}
 		// Write package path (namespace) metadata
 		if err := r.metaStringResolver.WriteMetaStringBytes(buffer, typeInfo.PkgPathBytes); err != nil {
 			return err
@@ -706,6 +835,109 @@ func (r *typeResolver) writeTypeInfo(buffer *ByteBuffer, typeInfo TypeInfo) erro
 	return nil
 }
 
+func (r *typeResolver) writeSharedTypeMeta(buffer *ByteBuffer, typeInfo TypeInfo) error {
+	context := r.fory.metaContext
+	typ := typeInfo.Type
+
+	if index, exists := context.typeMap[typ]; exists {
+		buffer.WriteVarUint32(index)
+		return nil
+	}
+
+	newIndex := uint32(len(context.typeMap))
+	buffer.WriteVarUint32(newIndex)
+	context.typeMap[typ] = newIndex
+
+	typeDef, err := r.getTypeDef(typeInfo.Type, true)
+	if err != nil {
+		return err
+	}
+	context.writingTypeDefs = append(context.writingTypeDefs, typeDef)
+	return nil
+}
+
+func (r *typeResolver) getTypeDef(typ reflect.Type, create bool) (*TypeDef, error) {
+	if existingTypeDef, exists := r.typeToTypeDef[typ]; exists {
+		return existingTypeDef, nil
+	}
+
+	if !create {
+		return nil, fmt.Errorf("TypeDef not found for type %s", typ)
+	}
+
+	// don't create TypeDef for pointer types, we create TypeDef for its element type instead.
+	if typ.Kind() == reflect.Ptr {
+		typ = typ.Elem()
+	}
+	zero := reflect.Zero(typ)
+	typeDef, err := buildTypeDef(r.fory, zero)
+	if err != nil {
+		return nil, err
+	}
+	r.typeToTypeDef[typ] = typeDef
+	return typeDef, nil
+}
+
+func (r *typeResolver) readSharedTypeMeta(buffer *ByteBuffer, value reflect.Value) (TypeInfo, error) {
+	context := r.fory.metaContext
+	index := buffer.ReadVarInt32() // shared meta index id
+	if index < 0 || index >= int32(len(context.readTypeInfos)) {
+		return TypeInfo{}, fmt.Errorf("TypeInfo not found for index %d", index)
+	}
+	info := context.readTypeInfos[index]
+	/*
+		todo: fix this logic for more elegant handle
+		There are two corner case:
+			1. value is pointer but read info not
+			2. value is not pointer but read info is pointer
+	*/
+	if value.Kind() == reflect.Ptr && IsNamespacedType(info.Serializer.TypeId()) {
+		info.Serializer = &ptrToStructSerializer{
+			type_:            value.Type().Elem(),
+			structSerializer: *info.Serializer.(*structSerializer),
+		}
+	} else if info.Type.Kind() == reflect.Ptr && value.Kind() != reflect.Ptr {
+		info.Type = info.Type.Elem()
+	}
+	return info, nil
+}
+
+func (r *typeResolver) writeTypeDefs(buffer *ByteBuffer) {
+	context := r.fory.metaContext
+	sz := len(context.writingTypeDefs)
+	buffer.WriteVarUint32Small7(uint32(sz))
+	for _, typeDef := range context.writingTypeDefs {
+		typeDef.writeTypeDef(buffer)
+	}
+	context.writingTypeDefs = nil
+}
+
+func (r *typeResolver) readTypeDefs(buffer *ByteBuffer) error {
+	numTypeDefs := buffer.ReadVarUint32Small7()
+	context := r.fory.metaContext
+	for i := 0; i < numTypeDefs; i++ {
+		id := buffer.ReadInt64()
+		var td *TypeDef
+		if existingTd, exists := r.defIdToTypeDef[id]; exists {
+			skipTypeDef(buffer, id)
+			td = existingTd
+		} else {
+			newTd, err := readTypeDef(r.fory, buffer, id)
+			if err != nil {
+				return err
+			}
+			r.defIdToTypeDef[id] = newTd
+			td = newTd
+		}
+		typeInfo, err := td.buildTypeInfo()
+		if err != nil {
+			return err
+		}
+		context.readTypeInfos = append(context.readTypeInfos, typeInfo)
+	}
+	return nil
+}
+
 func (r *typeResolver) createSerializer(type_ reflect.Type, mapInStruct bool) (s Serializer, err error) {
 	kind := type_.Kind()
 	switch kind {
@@ -720,6 +952,45 @@ func (r *typeResolver) createSerializer(type_ reflect.Type, mapInStruct bool) (s
 		return &ptrToValueSerializer{valueSerializer}, nil
 	case reflect.Slice:
 		elem := type_.Elem()
+		// Handle special slice types for xlang compatibility
+		if r.language == XLANG {
+			// Basic type slices should use array types for efficiency
+			switch elem.Kind() {
+			case reflect.Bool:
+				if type_ == boolSliceType {
+					return boolArraySerializer{}, nil
+				}
+			case reflect.Int8:
+				if type_ == int8SliceType {
+					return int8ArraySerializer{}, nil
+				}
+			case reflect.Int16:
+				if type_ == int16SliceType {
+					return int16ArraySerializer{}, nil
+				}
+			case reflect.Int32:
+				if type_ == int32SliceType {
+					return int32ArraySerializer{}, nil
+				}
+			case reflect.Int64:
+				if type_ == int64SliceType {
+					return int64ArraySerializer{}, nil
+				}
+			case reflect.Float32:
+				if type_ == float32SliceType {
+					return float32ArraySerializer{}, nil
+				}
+			case reflect.Float64:
+				if type_ == float64SliceType {
+					return float64ArraySerializer{}, nil
+				}
+			case reflect.Int, reflect.Uint:
+				// Platform-dependent types should use LIST for cross-platform compatibility
+				// We treat them as dynamic types to force LIST serialization
+				return sliceSerializer{}, nil
+			}
+		}
+		// For dynamic types or non-xlang mode, use generic slice serializer
 		if isDynamicType(elem) {
 			return sliceSerializer{}, nil
 		} else {
@@ -932,7 +1203,7 @@ func (r *typeResolver) readTypeByReadTag(buffer *ByteBuffer) (reflect.Type, erro
 	return r.typeTagToSerializers[metaString].(*ptrToStructSerializer).type_, err
 }
 
-func (r *typeResolver) readTypeInfo(buffer *ByteBuffer) (TypeInfo, error) {
+func (r *typeResolver) readTypeInfo(buffer *ByteBuffer, value reflect.Value) (TypeInfo, error) {
 	// Read variable-length type ID
 	typeID := buffer.ReadVarInt32()
 	internalTypeID := typeID // Extract lower 8 bits for internal type ID
@@ -940,6 +1211,9 @@ func (r *typeResolver) readTypeInfo(buffer *ByteBuffer) (TypeInfo, error) {
 		internalTypeID = -internalTypeID
 	}
 	if IsNamespacedType(TypeId(internalTypeID)) {
+		if r.metaShareEnabled() {
+			return r.readSharedTypeMeta(buffer, value)
+		}
 		// Read namespace and type name metadata bytes
 		nsBytes, err := r.metaStringResolver.ReadMetaStringBytes(buffer)
 		if err != nil {
@@ -1026,8 +1300,11 @@ func (r *typeResolver) getTypeById(id int16) (reflect.Type, error) {
 }
 
 func (r *typeResolver) getTypeInfoById(id int16) (TypeInfo, error) {
-	typeInfo := r.typeIDToTypeInfo[int32(id)]
-	return typeInfo, nil
+	if typeInfo, exists := r.typeIDToTypeInfo[int32(id)]; exists {
+		return typeInfo, nil
+	} else {
+		return TypeInfo{}, fmt.Errorf("typeInfo of typeID %d not found", id)
+	}
 }
 
 func (r *typeResolver) writeMetaString(buffer *ByteBuffer, str string) error {
@@ -1124,6 +1401,11 @@ func isPrimitiveType(typeID int16) bool {
 func isListType(typeID int16) bool {
 	return typeID == LIST
 }
+
+func isSetType(typeID int16) bool {
+	return typeID == SET
+}
+
 func isMapType(typeID int16) bool {
 	return typeID == MAP
 }
@@ -1161,3 +1443,14 @@ func getPrimitiveTypeSize(typeID int16) int {
 	}
 	return -1
 }
+
+func isUserDefinedType(typeID int16) bool {
+	return typeID == STRUCT ||
+		typeID == COMPATIBLE_STRUCT ||
+		typeID == NAMED_STRUCT ||
+		typeID == NAMED_COMPATIBLE_STRUCT ||
+		typeID == EXT ||
+		typeID == NAMED_EXT ||
+		typeID == ENUM ||
+		typeID == NAMED_ENUM
+}
