@@ -34,6 +34,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
@@ -83,14 +84,12 @@ public class ReflectionUtils {
       return null;
     }
     Constructor[] constructors = clazz.getDeclaredConstructors();
-    if (constructors.length == 0) {
-      return null;
-    } else {
-      return Stream.of(constructors)
-          .filter((c) -> c.getParameterCount() == 0)
-          .findAny()
-          .orElse(null);
+    Constructor ctr = null;
+    if (constructors.length != 0) {
+      ctr =
+          Stream.of(constructors).filter((c) -> c.getParameterCount() == 0).findAny().orElse(null);
     }
+    return ctr;
   }
 
   private static final ClassValue<MethodHandle> ctrHandleCache =
@@ -122,7 +121,11 @@ public class ReflectionUtils {
   public static MethodHandle getCtrHandle(Class<?> cls, boolean checked) {
     MethodHandle methodHandle = ctrHandleCache.get(cls);
     if (checked && methodHandle == null) {
-      throw new RuntimeException(String.format("Class %s doesn't have a no-arg constructor", cls));
+      throw new RuntimeException(
+          String.format(
+              "Class %s doesn't have a no-arg constructor, "
+                  + "please provide a no-arg constructor for the type",
+              cls));
     }
     return methodHandle;
   }
@@ -402,6 +405,12 @@ public class ReflectionUtils {
     return fields;
   }
 
+  public static List<Field> getSortedFields(Class<?> cls, boolean searchParent) {
+    List<Field> fields = getFields(cls, searchParent);
+    fields.sort(Comparator.comparing(f -> f.getName() + f.getDeclaringClass()));
+    return fields;
+  }
+
   /** Get fields values from provided object. */
   public static List<Object> getFieldValues(Collection<Field> fields, Object o) {
     List<Object> results = new ArrayList<>(fields.size());
