@@ -21,6 +21,7 @@ import datetime
 import enum
 import functools
 import logging
+import pickle
 import types
 from typing import TypeVar, Union
 from enum import Enum
@@ -54,25 +55,31 @@ from pyfory.serializer import (
     EnumSerializer,
     SliceSerializer,
     DataClassSerializer,
+    DataClassStubSerializer,
     StatefulSerializer,
     ReduceSerializer,
     FunctionSerializer,
     ObjectSerializer,
     TypeSerializer,
+    ModuleSerializer,
+    MappingProxySerializer,
     MethodSerializer,
     UnsupportedSerializer,
     NativeFuncMethodSerializer,
+    PickleBufferSerializer,
 )
 from pyfory.meta.metastring import MetaStringEncoder, MetaStringDecoder
+from pyfory.meta.meta_compressor import DeflaterMetaCompressor
 from pyfory.type import (
     TypeId,
-    Int8Type,
-    Int16Type,
-    Int32Type,
-    Int64Type,
-    Float32Type,
-    Float64Type,
+    int8,
+    int16,
+    int32,
+    int64,
+    float32,
+    float64,
     load_class,
+    is_struct_type,
     record_class_factory,
 )
 from pyfory._fory import (
@@ -80,6 +87,9 @@ from pyfory._fory import (
     # preserve 0 as flag for type id not set in TypeInfo`
     NO_TYPE_ID,
 )
+from pyfory.meta.typedef import TypeDef
+from pyfory.meta.typedef_decoder import decode_typedef, skip_typedef
+from pyfory.meta.typedef_encoder import encode_typedef
 
 try:
     import numpy as np
@@ -91,7 +101,7 @@ namespace_decoder = MetaStringDecoder(".", "_")
 typename_decoder = MetaStringDecoder("$", "_")
 
 if ENABLE_FORY_CYTHON_SERIALIZATION:
-    from pyfory._serialization import TypeInfo
+    from pyfory.serialization import TypeInfo
 else:
 
     class TypeInfo:
@@ -102,6 +112,7 @@ else:
             "namespace_bytes",
             "typename_bytes",
             "dynamic_type",
+            "type_def",
         )
 
         def __init__(
@@ -112,6 +123,7 @@ else:
             namespace_bytes=None,
             typename_bytes=None,
             dynamic_type: bool = False,
+            type_def: TypeDef = None,
         ):
             self.cls = cls
             self.type_id = type_id
@@ -119,6 +131,7 @@ else:
             self.namespace_bytes = namespace_bytes
             self.typename_bytes = typename_bytes
             self.dynamic_type = dynamic_type
+            self.type_def = type_def
 
         def __repr__(self):
             return f"TypeInfo(cls={self.cls}, type_id={self.type_id}, serializer={self.serializer})"
@@ -153,10 +166,14 @@ class TypeResolver:
         "namespace_decoder",
         "typename_encoder",
         "typename_decoder",
+        "meta_compressor",
         "require_registration",
         "metastring_resolver",
         "language",
         "_type_id_to_typeinfo",
+        "_meta_shared_typeinfo",
+        "meta_share",
+        "serialization_context",
         "_internal_py_serializer_map",
     )
 
@@ -164,7 +181,7 @@ class TypeResolver:
         self.fory = fory
         self.metastring_resolver = fory.metastring_resolver
         self.language = fory.language
-        self.require_registration = fory.require_type_registration
+        self.require_registration = fory.strict
         self._metastr_to_str = dict()
         self._metastr_to_type = dict()
         self._hash_to_metastring = dict()
@@ -180,8 +197,12 @@ class TypeResolver:
         self._named_type_to_typeinfo = dict()
         self.namespace_encoder = MetaStringEncoder(".", "_")
         self.namespace_decoder = MetaStringDecoder(".", "_")
+        # Cache for TypeDef and TypeInfo tuples (similar to Java's classIdToDef)
+        self._meta_shared_typeinfo = {}
         self.typename_encoder = MetaStringEncoder("$", "_")
         self.typename_decoder = MetaStringDecoder("$", "_")
+        self.meta_compressor = DeflaterMetaCompressor()
+        self.meta_share = meta_share
         self._internal_py_serializer_map = {}
 
     def initialize(self):
@@ -190,6 +211,7 @@ class TypeResolver:
             self._initialize_py()
         else:
             self._initialize_xlang()
+        self.serialization_context = self.fory.serialization_context
 
     def _initialize_py(self):
         register = functools.partial(self._register_type, internal=True)
@@ -198,15 +220,27 @@ class TypeResolver:
         register(slice, serializer=SliceSerializer)
         register(np.ndarray, serializer=NDArraySerializer)
         register(array.array, serializer=DynamicPyArraySerializer)
+        register(types.MappingProxyType, serializer=MappingProxySerializer)
+        register(pickle.PickleBuffer, serializer=PickleBufferSerializer)
         if not self.require_registration:
+            register(types.ModuleType, serializer=ModuleSerializer)
             self._internal_py_serializer_map = {
                 ReduceSerializer: (self._stub_cls("__Reduce__"), self._next_type_id()),
                 TypeSerializer: (self._stub_cls("__Type__"), self._next_type_id()),
                 MethodSerializer: (self._stub_cls("__Method__"), self._next_type_id()),
-                FunctionSerializer: (self._stub_cls("__Function__"), self._next_type_id()),
-                NativeFuncMethodSerializer: (self._stub_cls("__NativeFunction__"), self._next_type_id()),
+                FunctionSerializer: (
+                    self._stub_cls("__Function__"),
+                    self._next_type_id(),
+                ),
+                NativeFuncMethodSerializer: (
+                    self._stub_cls("__NativeFunction__"),
+                    self._next_type_id(),
+                ),
             }
-            for serializer, (stub_cls, type_id) in self._internal_py_serializer_map.items():
+            for serializer, (
+                stub_cls,
+                type_id,
+            ) in self._internal_py_serializer_map.items():
                 register(stub_cls, serializer=serializer, type_id=type_id)
 
     @staticmethod
@@ -222,18 +256,18 @@ class TypeResolver:
         register = functools.partial(self._register_type, internal=True)
         register(None, type_id=TypeId.NA, serializer=NoneSerializer)
         register(bool, type_id=TypeId.BOOL, serializer=BooleanSerializer)
-        register(Int8Type, type_id=TypeId.INT8, serializer=ByteSerializer)
-        register(Int16Type, type_id=TypeId.INT16, serializer=Int16Serializer)
-        register(Int32Type, type_id=TypeId.INT32, serializer=Int32Serializer)
-        register(Int64Type, type_id=TypeId.INT64, serializer=Int64Serializer)
+        register(int8, type_id=TypeId.INT8, serializer=ByteSerializer)
+        register(int16, type_id=TypeId.INT16, serializer=Int16Serializer)
+        register(int32, type_id=TypeId.INT32, serializer=Int32Serializer)
+        register(int64, type_id=TypeId.INT64, serializer=Int64Serializer)
         register(int, type_id=TypeId.INT64, serializer=Int64Serializer)
         register(
-            Float32Type,
+            float32,
             type_id=TypeId.FLOAT32,
             serializer=Float32Serializer,
         )
         register(
-            Float64Type,
+            float64,
             type_id=TypeId.FLOAT64,
             serializer=Float64Serializer,
         )
@@ -356,10 +390,14 @@ class TypeResolver:
                 serializer = EnumSerializer(self.fory, cls)
                 type_id = TypeId.NAMED_ENUM if type_id is None else ((type_id << 8) + TypeId.ENUM)
             else:
-                serializer = DataClassSerializer(self.fory, cls, xlang=True)
-                type_id = TypeId.NAMED_STRUCT if type_id is None else ((type_id << 8) + TypeId.STRUCT)
+                serializer = None
+                if self.meta_share:
+                    type_id = TypeId.NAMED_COMPATIBLE_STRUCT if type_id is None else ((type_id << 8) + TypeId.COMPATIBLE_STRUCT)
+                else:
+                    type_id = TypeId.NAMED_STRUCT if type_id is None else ((type_id << 8) + TypeId.STRUCT)
         elif not internal:
             type_id = TypeId.NAMED_EXT if type_id is None else ((type_id << 8) + TypeId.EXT)
+
         return self.__register_type(
             cls,
             type_id=type_id,
@@ -402,8 +440,17 @@ class TypeResolver:
         internal: bool = False,
     ):
         dynamic_type = type_id is not None and type_id < 0
-        if not internal and serializer is None:
+        # In metashare mode, for struct types, we want to keep serializer=None
+        # so that _set_typeinfo will be called to create the TypeDef-based serializer
+        should_create_serializer = (
+            not internal
+            and serializer is None
+            and not (self.meta_share and typename is not None and type_id is not None and is_struct_type(type_id & 0xFF))
+        )
+
+        if should_create_serializer:
             serializer = self._create_serializer(cls)
+
         if typename is None:
             typeinfo = TypeInfo(cls, type_id, serializer, None, None, dynamic_type)
         else:
@@ -460,7 +507,7 @@ class TypeResolver:
         type_info = self._types_info.get(cls)
         if type_info is not None:
             if type_info.serializer is None:
-                type_info.serializer = self._create_serializer(cls)
+                self._set_typeinfo(type_info)
             return type_info
         elif not create:
             return None
@@ -489,6 +536,24 @@ class TypeResolver:
             serializer=serializer,
         )
 
+    def _set_typeinfo(self, typeinfo):
+        type_id = typeinfo.type_id & 0xFF
+        if is_struct_type(type_id):
+            if self.meta_share:
+                type_def = encode_typedef(self, typeinfo.cls)
+                if type_def is not None:
+                    typeinfo.serializer = type_def.create_serializer(self)
+                    typeinfo.type_def = type_def
+                else:
+                    # Fallback to regular serializer
+                    typeinfo.serializer = DataClassSerializer(self.fory, typeinfo.cls, xlang=not self.fory.is_py)
+            else:
+                typeinfo.serializer = DataClassSerializer(self.fory, typeinfo.cls, xlang=not self.fory.is_py)
+        else:
+            typeinfo.serializer = self._create_serializer(typeinfo.cls)
+
+        return typeinfo
+
     def _create_serializer(self, cls):
         for clz in cls.__mro__:
             type_info = self._types_info.get(clz)
@@ -500,7 +565,8 @@ class TypeResolver:
                 # Use FunctionSerializer for function types (including lambdas)
                 serializer = FunctionSerializer(self.fory, cls)
             elif dataclasses.is_dataclass(cls):
-                serializer = DataClassSerializer(self.fory, cls)
+                # lazy create serializer to handle nested struct fields.
+                serializer = DataClassStubSerializer(self.fory, cls, xlang=not self.fory.is_py)
             elif issubclass(cls, enum.Enum):
                 serializer = EnumSerializer(self.fory, cls)
             elif ("builtin_function_or_method" in str(cls) or "cython_function_or_method" in str(cls)) and "<locals>" not in str(cls):
@@ -532,6 +598,28 @@ class TypeResolver:
                 serializer = UnsupportedSerializer(self.fory, cls)
         return serializer
 
+    def is_registered_by_name(self, cls):
+        typeinfo = self._types_info.get(cls)
+        if typeinfo is None:
+            return False
+        return TypeId.is_namespaced_type(typeinfo.type_id & 0xFF)
+
+    def is_registered_by_id(self, cls):
+        typeinfo = self._types_info.get(cls)
+        if typeinfo is None:
+            return False
+        return not TypeId.is_namespaced_type(typeinfo.type_id & 0xFF)
+
+    def get_registered_name(self, cls):
+        typeinfo = self._types_info.get(cls)
+        assert typeinfo is not None, f"{cls} not registered"
+        return typeinfo.decode_namespace(), typeinfo.decode_typename()
+
+    def get_registered_id(self, cls):
+        typeinfo = self._types_info.get(cls)
+        assert typeinfo is not None, f"{cls} not registered"
+        return typeinfo.type_id
+
     def _load_metabytes_to_typeinfo(self, ns_metabytes, type_metabytes):
         typeinfo = self._ns_type_to_typeinfo.get((ns_metabytes, type_metabytes))
         if typeinfo is not None:
@@ -551,6 +639,10 @@ class TypeResolver:
     def write_typeinfo(self, buffer, typeinfo):
         if typeinfo.dynamic_type:
             return
+        # Check if meta share is enabled first
+        if self.meta_share:
+            self.write_shared_type_meta(buffer, typeinfo)
+            return
         type_id = typeinfo.type_id
         internal_type_id = type_id & 0xFF
         buffer.write_varuint32(type_id)
@@ -559,6 +651,10 @@ class TypeResolver:
             self.metastring_resolver.write_meta_string_bytes(buffer, typeinfo.typename_bytes)
 
     def read_typeinfo(self, buffer):
+        # Check if meta share is enabled first
+        if self.meta_share:
+            return self.read_shared_type_meta(buffer)
+
         type_id = buffer.read_varuint32()
         internal_type_id = type_id & 0xFF
         if TypeId.is_namespaced_type(internal_type_id):
@@ -580,6 +676,82 @@ class TypeResolver:
         else:
             return self._type_id_to_typeinfo[type_id]
 
+    def get_typeinfo_by_id(self, type_id):
+        """Get typeinfo by type_id."""
+        return self._type_id_to_typeinfo[type_id]
+
+    def get_typeinfo_by_name(self, namespace, typename):
+        """Get typeinfo by namespace and typename."""
+        return self._named_type_to_typeinfo.get((namespace, typename))
+
+    def get_meta_compressor(self):
+        return self.meta_compressor
+
+    def write_shared_type_meta(self, buffer, typeinfo):
+        """Write shared type meta information."""
+        meta_context = self.fory.serialization_context.meta_context
+        meta_context.write_shared_typeinfo(buffer, typeinfo)
+
+    def read_shared_type_meta(self, buffer):
+        """Read shared type meta information."""
+        meta_context = self.serialization_context.meta_context
+        assert meta_context is not None, "Meta context must be set when meta share is enabled"
+        typeinfo = meta_context.read_shared_typeinfo(buffer)
+        return typeinfo
+
+    def write_type_defs(self, buffer):
+        """Write all type definitions that need to be sent."""
+        meta_context = self.fory.serialization_context.meta_context
+        if meta_context is None:
+            return
+        writing_type_defs = meta_context.get_writing_type_defs()
+        buffer.write_varuint32(len(writing_type_defs))
+        for type_def in writing_type_defs:
+            # Just copy the encoded bytes directly
+            buffer.write_bytes(type_def.encoded)
+
+    def read_type_defs(self, buffer):
+        """Read all type definitions from the buffer."""
+        meta_context = self.fory.serialization_context.meta_context
+        if meta_context is None:
+            return
+
+        num_type_defs = buffer.read_varuint32()
+        for i in range(num_type_defs):
+            # Read the header (first 8 bytes) to get the type ID
+            header = buffer.read_int64()
+            # Check if we already have this TypeDef cached
+            type_info = self._meta_shared_typeinfo.get(header)
+            if type_info is not None:
+                # Skip the rest of the TypeDef binary for faster performance
+                skip_typedef(buffer, header)
+            else:
+                # Read the TypeDef and create TypeInfo
+                type_def = decode_typedef(buffer, self, header=header)
+                type_info = self._build_type_info_from_typedef(type_def)
+                # Cache the tuple for future use
+                self._meta_shared_typeinfo[header] = type_info
+            meta_context.add_read_typeinfo(type_info)
+
+    def _build_type_info_from_typedef(self, type_def):
+        """Build TypeInfo from TypeDef using TypeDef's create_serializer method."""
+        # Create serializer using TypeDef's create_serializer method
+        serializer = type_def.create_serializer(self)
+        ns_metastr = self.namespace_encoder.encode(type_def.namespace or "")
+        ns_meta_bytes = self.metastring_resolver.get_metastr_bytes(ns_metastr)
+        type_metastr = self.typename_encoder.encode(type_def.typename)
+        type_meta_bytes = self.metastring_resolver.get_metastr_bytes(type_metastr)
+        typeinfo = TypeInfo(
+            type_def.cls,
+            type_def.type_id,
+            serializer,
+            ns_meta_bytes,
+            type_meta_bytes,
+            False,
+            type_def,
+        )
+        return typeinfo
+
     def reset(self):
         pass
 
