@@ -85,7 +85,7 @@ class Serializer(ABC):
         return False
 
 
-class CrossLanguageCompatibleSerializer(Serializer):
+class XlangCompatibleSerializer(Serializer):
     def __init__(self, fory, type_):
         super().__init__(fory, type_)
 
@@ -96,7 +96,7 @@ class CrossLanguageCompatibleSerializer(Serializer):
         return self.read(buffer)
 
 
-class BooleanSerializer(CrossLanguageCompatibleSerializer):
+class BooleanSerializer(XlangCompatibleSerializer):
     def write(self, buffer, value):
         buffer.write_bool(value)
 
@@ -104,7 +104,7 @@ class BooleanSerializer(CrossLanguageCompatibleSerializer):
         return buffer.read_bool()
 
 
-class ByteSerializer(CrossLanguageCompatibleSerializer):
+class ByteSerializer(XlangCompatibleSerializer):
     def write(self, buffer, value):
         buffer.write_int8(value)
 
@@ -112,7 +112,7 @@ class ByteSerializer(CrossLanguageCompatibleSerializer):
         return buffer.read_int8()
 
 
-class Int16Serializer(CrossLanguageCompatibleSerializer):
+class Int16Serializer(XlangCompatibleSerializer):
     def write(self, buffer, value):
         buffer.write_int16(value)
 
@@ -120,7 +120,7 @@ class Int16Serializer(CrossLanguageCompatibleSerializer):
         return buffer.read_int16()
 
 
-class Int32Serializer(CrossLanguageCompatibleSerializer):
+class Int32Serializer(XlangCompatibleSerializer):
     def write(self, buffer, value):
         buffer.write_varint32(value)
 
@@ -142,7 +142,7 @@ class Int64Serializer(Serializer):
         return buffer.read_varint64()
 
 
-class Float32Serializer(CrossLanguageCompatibleSerializer):
+class Float32Serializer(XlangCompatibleSerializer):
     def write(self, buffer, value):
         buffer.write_float(value)
 
@@ -150,7 +150,7 @@ class Float32Serializer(CrossLanguageCompatibleSerializer):
         return buffer.read_float()
 
 
-class Float64Serializer(CrossLanguageCompatibleSerializer):
+class Float64Serializer(XlangCompatibleSerializer):
     def write(self, buffer, value):
         buffer.write_double(value)
 
@@ -158,7 +158,7 @@ class Float64Serializer(CrossLanguageCompatibleSerializer):
         return buffer.read_double()
 
 
-class StringSerializer(CrossLanguageCompatibleSerializer):
+class StringSerializer(XlangCompatibleSerializer):
     def __init__(self, fory, type_):
         super().__init__(fory, type_)
         self.need_to_write_ref = False
@@ -173,7 +173,7 @@ class StringSerializer(CrossLanguageCompatibleSerializer):
 _base_date = datetime.date(1970, 1, 1)
 
 
-class DateSerializer(CrossLanguageCompatibleSerializer):
+class DateSerializer(XlangCompatibleSerializer):
     def write(self, buffer, value: datetime.date):
         if not isinstance(value, datetime.date):
             raise TypeError("{} should be {} instead of {}".format(value, datetime.date, type(value)))
@@ -185,7 +185,7 @@ class DateSerializer(CrossLanguageCompatibleSerializer):
         return _base_date + datetime.timedelta(days=days)
 
 
-class TimestampSerializer(CrossLanguageCompatibleSerializer):
+class TimestampSerializer(XlangCompatibleSerializer):
     __win_platform = platform.system() == "Windows"
 
     def _get_timestamp(self, value: datetime.datetime):
@@ -208,11 +208,15 @@ class TimestampSerializer(CrossLanguageCompatibleSerializer):
         return datetime.datetime.fromtimestamp(ts)
 
 
-COLLECTION_DEFAULT_FLAG = 0b0
-COLLECTION_TRACKING_REF = 0b1
-COLLECTION_HAS_NULL = 0b10
-COLLECTION_NOT_DECL_ELEMENT_TYPE = 0b100
-COLLECTION_NOT_SAME_TYPE = 0b1000
+COLL_DEFAULT_FLAG = 0b0
+COLL_TRACKING_REF = 0b1
+COLL_HAS_NULL = 0b10
+COLL_IS_DECL_ELEMENT_TYPE = 0b100
+COLL_IS_SAME_TYPE = 0b1000
+COLL_DECL_SAME_TYPE_TRACKING_REF = COLL_IS_DECL_ELEMENT_TYPE | COLL_IS_SAME_TYPE | COLL_TRACKING_REF
+COLL_DECL_SAME_TYPE_NOT_TRACKING_REF = COLL_IS_DECL_ELEMENT_TYPE | COLL_IS_SAME_TYPE
+COLL_DECL_SAME_TYPE_HAS_NULL = COLL_IS_DECL_ELEMENT_TYPE | COLL_IS_SAME_TYPE | COLL_HAS_NULL
+COLL_DECL_SAME_TYPE_NOT_HAS_NULL = COLL_IS_DECL_ELEMENT_TYPE | COLL_IS_SAME_TYPE
 
 
 class CollectionSerializer(Serializer):
@@ -242,40 +246,42 @@ class CollectionSerializer(Serializer):
         self.is_py = fory.is_py
 
     def write_header(self, buffer, value):
-        collect_flag = COLLECTION_DEFAULT_FLAG
+        collect_flag = COLL_DEFAULT_FLAG
         elem_type = self.elem_type
         elem_typeinfo = self.elem_typeinfo
         has_null = False
-        has_different_type = False
+        has_same_type = True
         if elem_type is None:
-            collect_flag |= COLLECTION_NOT_DECL_ELEMENT_TYPE
             for s in value:
                 if not has_null and s is None:
                     has_null = True
                     continue
                 if elem_type is None:
                     elem_type = type(s)
-                elif not has_different_type and type(s) is not elem_type:
-                    collect_flag |= COLLECTION_NOT_SAME_TYPE
-                    has_different_type = True
-            if not has_different_type and elem_type is not None:
-                elem_typeinfo = self.type_resolver.get_typeinfo(elem_type)
+                elif has_same_type and type(s) is not elem_type:
+                    has_same_type = False
+            if has_same_type:
+                collect_flag |= COLL_IS_SAME_TYPE
+                if elem_type is not None:
+                    elem_typeinfo = self.type_resolver.get_typeinfo(elem_type)
         else:
+            collect_flag |= COLL_IS_DECL_ELEMENT_TYPE | COLL_IS_SAME_TYPE
             for s in value:
                 if s is None:
                     has_null = True
                     break
+
         if has_null:
-            collect_flag |= COLLECTION_HAS_NULL
+            collect_flag |= COLL_HAS_NULL
         if self.fory.ref_tracking:
             if self.elem_tracking_ref == 1:
-                collect_flag |= COLLECTION_TRACKING_REF
+                collect_flag |= COLL_TRACKING_REF
             elif self.elem_tracking_ref == -1:
-                if has_different_type or elem_typeinfo.serializer.need_to_write_ref:
-                    collect_flag |= COLLECTION_TRACKING_REF
+                if not has_same_type or elem_typeinfo.serializer.need_to_write_ref:
+                    collect_flag |= COLL_TRACKING_REF
         buffer.write_varuint32(len(value))
         buffer.write_int8(collect_flag)
-        if not has_different_type and (collect_flag & COLLECTION_NOT_DECL_ELEMENT_TYPE) != 0:
+        if has_same_type and (collect_flag & COLL_IS_DECL_ELEMENT_TYPE) == 0:
             self.type_resolver.write_typeinfo(buffer, elem_typeinfo)
         return collect_flag, elem_typeinfo
 
@@ -284,8 +290,8 @@ class CollectionSerializer(Serializer):
             buffer.write_varuint32(0)
             return
         collect_flag, typeinfo = self.write_header(buffer, value)
-        if (collect_flag & COLLECTION_NOT_SAME_TYPE) == 0:
-            if (collect_flag & COLLECTION_TRACKING_REF) == 0:
+        if (collect_flag & COLL_IS_SAME_TYPE) != 0:
+            if (collect_flag & COLL_TRACKING_REF) == 0:
                 self._write_same_type_no_ref(buffer, value, typeinfo)
             else:
                 self._write_same_type_ref(buffer, value, typeinfo)
@@ -326,12 +332,12 @@ class CollectionSerializer(Serializer):
         if len_ == 0:
             return collection_
         collect_flag = buffer.read_int8()
-        if (collect_flag & COLLECTION_NOT_SAME_TYPE) == 0:
-            if collect_flag & COLLECTION_NOT_DECL_ELEMENT_TYPE != 0:
+        if (collect_flag & COLL_IS_SAME_TYPE) != 0:
+            if collect_flag & COLL_IS_DECL_ELEMENT_TYPE == 0:
                 typeinfo = self.type_resolver.read_typeinfo(buffer)
             else:
                 typeinfo = self.elem_typeinfo
-            if (collect_flag & COLLECTION_TRACKING_REF) == 0:
+            if (collect_flag & COLL_TRACKING_REF) == 0:
                 self._read_same_type_no_ref(buffer, len_, collection_, typeinfo)
             else:
                 self._read_same_type_ref(buffer, len_, collection_, typeinfo)
@@ -458,7 +464,7 @@ class MapSerializer(Serializer):
         items_iter = iter(obj.items())
         key, value = next(items_iter)
         has_next = True
-        serialize_ref = fory.serialize_ref if self.fory.is_py else fory.xserialize_ref
+        write_ref = fory.write_ref if self.fory.is_py else fory.xwrite_ref
         while has_next:
             while True:
                 if key is not None:
@@ -474,7 +480,7 @@ class MapSerializer(Serializer):
                             self._write_obj(key_serializer, buffer, key)
                     else:
                         buffer.write_int8(VALUE_HAS_NULL | TRACKING_KEY_REF)
-                        serialize_ref(buffer, key)
+                        write_ref(buffer, key)
                 else:
                     if value is not None:
                         if value_serializer is not None:
@@ -489,7 +495,7 @@ class MapSerializer(Serializer):
                                 value_serializer.write(buffer, value)
                         else:
                             buffer.write_int8(KEY_HAS_NULL | TRACKING_VALUE_REF)
-                            serialize_ref(buffer, value)
+                            write_ref(buffer, value)
                     else:
                         buffer.write_int8(KV_NULL)
                 try:
@@ -561,7 +567,7 @@ class MapSerializer(Serializer):
         if size != 0:
             chunk_header = buffer.read_uint8()
         key_serializer, value_serializer = self.key_serializer, self.value_serializer
-        deserialize_ref = fory.deserialize_ref if self.fory.is_py else fory.xdeserialize_ref
+        read_ref = fory.read_ref if self.fory.is_py else fory.xread_ref
         fory.inc_depth()
         while size > 0:
             while True:
@@ -583,7 +589,7 @@ class MapSerializer(Serializer):
                             else:
                                 key = self._read_obj(key_serializer, buffer)
                         else:
-                            key = deserialize_ref(buffer)
+                            key = read_ref(buffer)
                         map_[key] = None
                 else:
                     if not value_has_null:
@@ -597,7 +603,7 @@ class MapSerializer(Serializer):
                                     value = self._read_obj(value_serializer, buffer)
                                     ref_resolver.set_read_object(ref_id, value)
                         else:
-                            value = deserialize_ref(buffer)
+                            value = read_ref(buffer)
                         map_[None] = value
                     else:
                         map_[None] = None
@@ -701,7 +707,7 @@ class SliceSerializer(Serializer):
                 buffer.write_int8(NULL_FLAG)
             else:
                 buffer.write_int8(NOT_NULL_VALUE_FLAG)
-                self.fory.serialize_nonref(buffer, start)
+                self.fory.write_no_ref(buffer, start)
         if type(stop) is int:
             # TODO support varint128
             buffer.write_int16(NOT_NULL_INT64_FLAG)
@@ -711,7 +717,7 @@ class SliceSerializer(Serializer):
                 buffer.write_int8(NULL_FLAG)
             else:
                 buffer.write_int8(NOT_NULL_VALUE_FLAG)
-                self.fory.serialize_nonref(buffer, stop)
+                self.fory.write_no_ref(buffer, stop)
         if type(step) is int:
             # TODO support varint128
             buffer.write_int16(NOT_NULL_INT64_FLAG)
@@ -721,21 +727,21 @@ class SliceSerializer(Serializer):
                 buffer.write_int8(NULL_FLAG)
             else:
                 buffer.write_int8(NOT_NULL_VALUE_FLAG)
-                self.fory.serialize_nonref(buffer, step)
+                self.fory.write_no_ref(buffer, step)
 
     def read(self, buffer):
         if buffer.read_int8() == NULL_FLAG:
             start = None
         else:
-            start = self.fory.deserialize_nonref(buffer)
+            start = self.fory.read_no_ref(buffer)
         if buffer.read_int8() == NULL_FLAG:
             stop = None
         else:
-            stop = self.fory.deserialize_nonref(buffer)
+            stop = self.fory.read_no_ref(buffer)
         if buffer.read_int8() == NULL_FLAG:
             step = None
         else:
-            step = self.fory.deserialize_nonref(buffer)
+            step = self.fory.read_no_ref(buffer)
         return slice(start, stop, step)
 
     def xwrite(self, buffer, value):
