@@ -26,6 +26,7 @@ use crate::types;
 use crate::types::{is_user_type, RefFlag};
 use chrono::{NaiveDate, NaiveDateTime};
 use std::rc::Rc;
+use std::time::Duration;
 
 #[allow(unreachable_code)]
 pub fn skip_field_value(
@@ -381,6 +382,9 @@ fn skip_value(
         types::TIMESTAMP => {
             <NaiveDateTime as Serializer>::fory_read_data(context)?;
         }
+        types::DURATION => {
+            <Duration as Serializer>::fory_read_data(context)?;
+        }
         types::BINARY => {
             <Vec<u8> as Serializer>::fory_read_data(context)?;
         }
@@ -420,6 +424,9 @@ fn skip_value(
         types::USIZE => {
             <usize as Serializer>::fory_read_data(context)?;
         }
+        types::U128 => {
+            <u128 as Serializer>::fory_read_data(context)?;
+        }
         types::U16_ARRAY => {
             <Vec<u16> as Serializer>::fory_read_data(context)?;
         }
@@ -432,6 +439,9 @@ fn skip_value(
         types::USIZE_ARRAY => {
             <Vec<usize> as Serializer>::fory_read_data(context)?;
         }
+        types::U128_ARRAY => {
+            <Vec<u128> as Serializer>::fory_read_data(context)?;
+        }
 
         // Container types
         types::LIST | types::SET => {
