@@ -21,8 +21,8 @@ use crate::resolver::context::{ReadContext, WriteContext};
 use crate::serializer::Serializer;
 use crate::types::TypeId;
 use crate::types::{
-    is_user_type, BOOL, ENUM, FLOAT32, FLOAT64, INT16, INT32, INT64, INT8, NAMED_ENUM, U16, U32,
-    U64, U8, USIZE,
+    is_user_type, BOOL, ENUM, FLOAT32, FLOAT64, INT16, INT32, INT64, INT8, NAMED_ENUM, U128, U16,
+    U32, U64, U8, USIZE,
 };
 
 #[inline(always)]
@@ -70,7 +70,18 @@ pub const fn field_need_write_ref_into(type_id: u32, nullable: bool) -> bool {
     let internal_type_id = type_id & 0xff;
     !matches!(
         internal_type_id,
-        BOOL | INT8 | INT16 | INT32 | INT64 | FLOAT32 | FLOAT64 | U8 | U16 | U32 | U64 | USIZE
+        BOOL | INT8
+            | INT16
+            | INT32
+            | INT64
+            | FLOAT32
+            | FLOAT64
+            | U8
+            | U16
+            | U32
+            | U64
+            | USIZE
+            | U128
     )
 }
 
