@@ -86,14 +86,16 @@ pub enum TypeId {
     U32 = 66,
     U64 = 67,
     USIZE = 68,
-    VAR_U32 = 69,
-    VAR_U64 = 70,
-    SLI_U64 = 71,
-    U16_ARRAY = 72,
-    U32_ARRAY = 73,
-    U64_ARRAY = 74,
-    USIZE_ARRAY = 75,
-    UNKNOWN = 76,
+    U128 = 69,
+    VAR_U32 = 70,
+    VAR_U64 = 71,
+    SLI_U64 = 72,
+    U16_ARRAY = 73,
+    U32_ARRAY = 74,
+    U64_ARRAY = 75,
+    USIZE_ARRAY = 76,
+    U128_ARRAY = 77,
+    UNKNOWN = 78,
 }
 
 pub const BOOL: u32 = TypeId::BOOL as u32;
@@ -140,6 +142,7 @@ pub const U16: u32 = TypeId::U16 as u32;
 pub const U32: u32 = TypeId::U32 as u32;
 pub const U64: u32 = TypeId::U64 as u32;
 pub const USIZE: u32 = TypeId::USIZE as u32;
+pub const U128: u32 = TypeId::U128 as u32;
 pub const VAR_U32: u32 = TypeId::VAR_U32 as u32;
 pub const VAR_U64: u32 = TypeId::VAR_U64 as u32;
 pub const SLI_U64: u32 = TypeId::SLI_U64 as u32;
@@ -147,6 +150,7 @@ pub const U16_ARRAY: u32 = TypeId::U16_ARRAY as u32;
 pub const U32_ARRAY: u32 = TypeId::U32_ARRAY as u32;
 pub const U64_ARRAY: u32 = TypeId::U64_ARRAY as u32;
 pub const USIZE_ARRAY: u32 = TypeId::USIZE_ARRAY as u32;
+pub const U128_ARRAY: u32 = TypeId::U128_ARRAY as u32;
 pub const UNKNOWN: u32 = TypeId::UNKNOWN as u32;
 
 const MAX_UNT32: u64 = (1 << 31) - 1;
@@ -164,7 +168,7 @@ pub fn compute_string_hash(s: &str) -> u32 {
     hash as u32
 }
 
-pub static BASIC_TYPES: [TypeId; 27] = [
+pub static BASIC_TYPES: [TypeId; 29] = [
     TypeId::BOOL,
     TypeId::INT8,
     TypeId::INT16,
@@ -188,13 +192,15 @@ pub static BASIC_TYPES: [TypeId; 27] = [
     TypeId::U32,
     TypeId::U64,
     TypeId::USIZE,
+    TypeId::U128,
     TypeId::U16_ARRAY,
     TypeId::U32_ARRAY,
     TypeId::U64_ARRAY,
     TypeId::USIZE_ARRAY,
+    TypeId::U128_ARRAY,
 ];
 
-pub static PRIMITIVE_TYPES: [u32; 12] = [
+pub static PRIMITIVE_TYPES: [u32; 13] = [
     TypeId::BOOL as u32,
     TypeId::INT8 as u32,
     TypeId::INT16 as u32,
@@ -207,9 +213,10 @@ pub static PRIMITIVE_TYPES: [u32; 12] = [
     TypeId::U32 as u32,
     TypeId::U64 as u32,
     TypeId::USIZE as u32,
+    TypeId::U128 as u32,
 ];
 
-pub static PRIMITIVE_ARRAY_TYPES: [u32; 12] = [
+pub static PRIMITIVE_ARRAY_TYPES: [u32; 13] = [
     TypeId::BOOL_ARRAY as u32,
     TypeId::BINARY as u32,
     TypeId::INT8_ARRAY as u32,
@@ -221,10 +228,11 @@ pub static PRIMITIVE_ARRAY_TYPES: [u32; 12] = [
     TypeId::U16_ARRAY as u32,
     TypeId::U32_ARRAY as u32,
     TypeId::U64_ARRAY as u32,
-    TypeId::U64_ARRAY as u32,
+    TypeId::USIZE_ARRAY as u32,
+    TypeId::U128_ARRAY as u32,
 ];
 
-pub static BASIC_TYPE_NAMES: [&str; 15] = [
+pub static BASIC_TYPE_NAMES: [&str; 16] = [
     "bool",
     "i8",
     "i16",
@@ -240,6 +248,7 @@ pub static BASIC_TYPE_NAMES: [&str; 15] = [
     "u32",
     "u64",
     "usize",
+    "u128",
 ];
 
 pub static CONTAINER_TYPES: [TypeId; 3] = [TypeId::LIST, TypeId::SET, TypeId::MAP];
@@ -259,6 +268,7 @@ pub static PRIMITIVE_ARRAY_TYPE_MAP: &[(&str, u32, &str)] = &[
     ("u32", TypeId::U32_ARRAY as u32, "Vec<u32>"),
     ("u64", TypeId::U64_ARRAY as u32, "Vec<u64>"),
     ("usize", TypeId::USIZE_ARRAY as u32, "Vec<usize>"),
+    ("u128", TypeId::U128_ARRAY as u32, "Vec<u128>"),
 ];
 
 /// Keep as const fn for compile time evaluation or constant folding
@@ -278,6 +288,7 @@ pub const fn is_primitive_type_id(type_id: TypeId) -> bool {
             | TypeId::U32
             | TypeId::U64
             | TypeId::USIZE
+            | TypeId::U128
     )
 }
 
