@@ -21,8 +21,8 @@ use std::sync::atomic::{AtomicU32, Ordering};
 use syn::Field;
 
 use super::util::{
-    classify_trait_object_field, generic_tree_to_tokens, get_sort_fields_ts, parse_generic_tree,
-    StructField,
+    classify_trait_object_field, generic_tree_to_tokens, get_filtered_fields_iter,
+    get_sort_fields_ts, parse_generic_tree, StructField,
 };
 
 // Global type ID counter that auto-grows from 0 at macro processing time
@@ -72,7 +72,7 @@ pub fn gen_get_sorted_field_names(fields: &[&Field]) -> TokenStream {
 }
 
 pub fn gen_field_fields_info(fields: &[&Field]) -> TokenStream {
-    let field_infos = fields.iter().map(|field| {
+    let field_infos = get_filtered_fields_iter(fields).map(|field| {
         let ty = &field.ty;
         let name = format!("{}", field.ident.as_ref().expect("should be field name"));
         match classify_trait_object_field(ty) {
@@ -125,8 +125,14 @@ pub fn gen_field_fields_info(fields: &[&Field]) -> TokenStream {
             }
         }
     });
+
+    // Get sorted field names for sorting
+    let static_field_names = get_sort_fields_ts(fields);
+
     quote! {
-        let field_infos: Vec<fory_core::meta::FieldInfo> = vec![#(#field_infos),*];
+        let mut field_infos: Vec<fory_core::meta::FieldInfo> = vec![#(#field_infos),*];
+        let sorted_field_names = #static_field_names;
+        fory_core::meta::sort_fields(&mut field_infos, sorted_field_names)?;
         Ok(field_infos)
     }
 }
