@@ -20,12 +20,12 @@ use crate::util::{
     CollectionTraitInfo,
 };
 use fory_core::types::{TypeId, PRIMITIVE_ARRAY_TYPE_MAP};
+use fory_core::util::ENABLE_FORY_DEBUG_OUTPUT;
 use proc_macro2::{Ident, TokenStream};
 use quote::{format_ident, quote, ToTokens};
 use std::cell::RefCell;
 use std::collections::HashMap;
 use std::fmt;
-use std::sync::OnceLock;
 use syn::{Field, GenericArgument, PathArguments, Type};
 
 thread_local! {
@@ -57,19 +57,6 @@ pub(super) fn get_struct_name() -> Option<String> {
     MACRO_CONTEXT.with(|ctx| ctx.borrow().as_ref().map(|c| c.struct_name.clone()))
 }
 
-/// Global flag to check if ENABLE_FORY_DEBUG_OUTPUT environment variable is set.
-static ENABLE_FORY_DEBUG_OUTPUT: OnceLock<bool> = OnceLock::new();
-
-/// Check if ENABLE_FORY_DEBUG_OUTPUT environment variable is set.
-#[inline]
-fn enable_debug_output() -> bool {
-    *ENABLE_FORY_DEBUG_OUTPUT.get_or_init(|| {
-        std::env::var("ENABLE_FORY_DEBUG_OUTPUT")
-            .map(|v| v == "1" || v.eq_ignore_ascii_case("true"))
-            .unwrap_or(false)
-    })
-}
-
 pub(super) fn is_debug_enabled() -> bool {
     MACRO_CONTEXT.with(|ctx| {
         ctx.borrow()
@@ -242,6 +229,8 @@ pub(super) fn classify_trait_object_field(ty: &Type) -> StructField {
 pub(super) struct TypeNode {
     pub name: String,
     pub generics: Vec<TypeNode>,
+    /// For arrays, store the original type string "[T; N]" to preserve length info
+    pub original_type_str: Option<String>,
 }
 
 pub(super) fn try_primitive_vec_type(node: &TypeNode) -> Option<TokenStream> {
@@ -290,6 +279,14 @@ impl fmt::Display for TypeNode {
                     .collect::<Vec<_>>()
                     .join(", ")
             )
+        } else if self.name == "Array" {
+            // For arrays, use the original type string if available (e.g., "[f32; 4]")
+            if let Some(ref original) = self.original_type_str {
+                write!(f, "{}", original)
+            } else {
+                // Fallback - shouldn't happen if properly constructed
+                write!(f, "Array")
+            }
         } else if self.generics.is_empty() {
             write!(f, "{}", self.name)
         } else {
@@ -339,6 +336,7 @@ pub(super) fn parse_generic_tree(ty: &Type) -> TypeNode {
         return TypeNode {
             name: "TraitObject".to_string(),
             generics: vec![],
+            original_type_str: None,
         };
     }
 
@@ -347,15 +345,19 @@ pub(super) fn parse_generic_tree(ty: &Type) -> TypeNode {
         return TypeNode {
             name: "Tuple".to_string(),
             generics: vec![],
+            original_type_str: None,
         };
     }
 
-    // Handle arrays - extract element type
+    // Handle arrays - extract element type and preserve original type string
     if let Type::Array(array) = ty {
         let elem_node = parse_generic_tree(&array.elem);
+        // Preserve the original type string including the length (e.g., "[f32; 4]")
+        let original_type_str = quote!(#ty).to_string().replace(' ', "");
         return TypeNode {
             name: "Array".to_string(),
             generics: vec![elem_node],
+            original_type_str: Some(original_type_str),
         };
     }
 
@@ -381,7 +383,11 @@ pub(super) fn parse_generic_tree(ty: &Type) -> TypeNode {
     } else {
         vec![]
     };
-    TypeNode { name, generics }
+    TypeNode {
+        name,
+        generics,
+        original_type_str: None,
+    }
 }
 
 pub(super) fn generic_tree_to_tokens(node: &TypeNode) -> TokenStream {
@@ -421,6 +427,7 @@ pub(super) fn generic_tree_to_tokens(node: &TypeNode) -> TokenStream {
                     "u32" => quote! { fory_core::types::TypeId::U32_ARRAY as u32 },
                     "u64" => quote! { fory_core::types::TypeId::U64_ARRAY as u32 },
                     "usize" => quote! { fory_core::types::TypeId::USIZE_ARRAY as u32 },
+                    "u128" => quote! { fory_core::types::TypeId::U128_ARRAY as u32 },
                     _ => quote! { fory_core::types::TypeId::LIST as u32 },
                 };
                 return quote! {
@@ -541,8 +548,8 @@ fn extract_option_inner(s: &str) -> Option<&str> {
     s.strip_prefix("Option<")?.strip_suffix(">")
 }
 
-const PRIMITIVE_TYPE_NAMES: [&str; 12] = [
-    "bool", "i8", "i16", "i32", "i64", "f32", "f64", "u8", "u16", "u32", "u64", "usize",
+const PRIMITIVE_TYPE_NAMES: [&str; 13] = [
+    "bool", "i8", "i16", "i32", "i64", "f32", "f64", "u8", "u16", "u32", "u64", "usize", "u128",
 ];
 
 fn get_primitive_type_id(ty: &str) -> u32 {
@@ -559,6 +566,7 @@ fn get_primitive_type_id(ty: &str) -> u32 {
         "u32" => TypeId::U32 as u32,
         "u64" => TypeId::U64 as u32,
         "usize" => TypeId::USIZE as u32,
+        "u128" => TypeId::U128 as u32,
         _ => unreachable!("Unknown primitive type: {}", ty),
     }
 }
@@ -567,6 +575,60 @@ pub(super) fn is_primitive_type(ty: &str) -> bool {
     PRIMITIVE_TYPE_NAMES.contains(&ty)
 }
 
+/// Mapping of primitive type names to their writer and reader method names
+/// Order: (type_name, writer_method, reader_method)
+static PRIMITIVE_IO_METHODS: &[(&str, &str, &str)] = &[
+    ("bool", "write_bool", "read_bool"),
+    ("i8", "write_i8", "read_i8"),
+    ("i16", "write_i16", "read_i16"),
+    ("i32", "write_varint32", "read_varint32"),
+    ("i64", "write_varint64", "read_varint64"),
+    ("f32", "write_f32", "read_f32"),
+    ("f64", "write_f64", "read_f64"),
+    ("u8", "write_u8", "read_u8"),
+    ("u16", "write_u16", "read_u16"),
+    ("u32", "write_u32", "read_u32"),
+    ("u64", "write_u64", "read_u64"),
+    ("usize", "write_usize", "read_usize"),
+    ("u128", "write_u128", "read_u128"),
+];
+
+/// Check if a type is a direct primitive numeric type (not wrapped in Option, Vec, etc.)
+pub(super) fn is_direct_primitive_numeric_type(ty: &Type) -> bool {
+    if let Type::Path(type_path) = ty {
+        if let Some(seg) = type_path.path.segments.last() {
+            // Check if it's a simple type path without generics
+            if matches!(seg.arguments, PathArguments::None) {
+                let type_name = seg.ident.to_string();
+                return PRIMITIVE_IO_METHODS
+                    .iter()
+                    .any(|(name, _, _)| *name == type_name.as_str());
+            }
+        }
+    }
+    false
+}
+
+/// Get the writer method name for a primitive numeric type
+/// Panics if type_name is not a primitive type
+pub(super) fn get_primitive_writer_method(type_name: &str) -> &'static str {
+    PRIMITIVE_IO_METHODS
+        .iter()
+        .find(|(name, _, _)| *name == type_name)
+        .map(|(_, writer, _)| *writer)
+        .unwrap_or_else(|| panic!("type_name '{}' must be a primitive type", type_name))
+}
+
+/// Get the reader method name for a primitive numeric type
+/// Panics if type_name is not a primitive type
+pub(super) fn get_primitive_reader_method(type_name: &str) -> &'static str {
+    PRIMITIVE_IO_METHODS
+        .iter()
+        .find(|(name, _, _)| *name == type_name)
+        .map(|(_, _, reader)| *reader)
+        .unwrap_or_else(|| panic!("type_name '{}' must be a primitive type", type_name))
+}
+
 pub(crate) fn get_type_id_by_type_ast(ty: &Type) -> u32 {
     let ty_str: String = ty
         .to_token_stream()
@@ -614,6 +676,7 @@ pub(crate) fn get_type_id_by_name(ty: &str) -> u32 {
         "Vec<u32>" => return TypeId::U32_ARRAY as u32,
         "Vec<u64>" => return TypeId::U64_ARRAY as u32,
         "Vec<usize>" => return TypeId::USIZE_ARRAY as u32,
+        "Vec<u128>" => return TypeId::U128_ARRAY as u32,
         _ => {}
     }
 
@@ -635,6 +698,7 @@ pub(crate) fn get_type_id_by_name(ty: &str) -> u32 {
                 "u32" => return TypeId::U32_ARRAY as u32,
                 "u64" => return TypeId::U64_ARRAY as u32,
                 "usize" => return TypeId::USIZE_ARRAY as u32,
+                "u128" => return TypeId::U128_ARRAY as u32,
                 _ => {
                     // Non-primitive array elements, treat as LIST
                     return TypeId::LIST as u32;
@@ -687,6 +751,7 @@ fn get_primitive_type_size(type_id_num: u32) -> i32 {
         TypeId::U32 => 4,
         TypeId::U64 => 8,
         TypeId::USIZE => 8,
+        TypeId::U128 => 16,
         _ => unreachable!(),
     }
 }
@@ -717,6 +782,11 @@ fn is_internal_type_id(type_id: u32) -> bool {
         TypeId::FLOAT16_ARRAY as u32,
         TypeId::FLOAT32_ARRAY as u32,
         TypeId::FLOAT64_ARRAY as u32,
+        TypeId::U16_ARRAY as u32,
+        TypeId::U32_ARRAY as u32,
+        TypeId::U64_ARRAY as u32,
+        TypeId::USIZE_ARRAY as u32,
+        TypeId::U128_ARRAY as u32,
     ]
     .contains(&type_id)
 }
@@ -941,7 +1011,7 @@ pub(crate) fn compute_struct_version_hash(fields: &[&Field]) -> i32 {
     let version = (hash & 0xFFFF_FFFF) as u32;
     let version = version as i32;
 
-    if enable_debug_output() {
+    if ENABLE_FORY_DEBUG_OUTPUT {
         if let Some(struct_name) = get_struct_name() {
             println!(
                 "[fory-debug] struct {struct_name} version fingerprint=\"{fingerprint}\" hash={version}"
