@@ -188,7 +188,7 @@ fn nonexistent_struct() {
 #[test]
 fn option() {
     #[derive(ForyObject, Debug, PartialEq)]
-    #[fory_debug]
+    #[fory(debug)]
     struct Animal {
         f1: Option<String>,
         f2: Option<String>,
@@ -332,7 +332,7 @@ fn nullable_container() {
 #[test]
 fn inner_nullable() {
     #[derive(ForyObject, Debug)]
-    #[fory_debug]
+    #[fory(debug)]
     pub struct Item1 {
         f1: Vec<Option<String>>,
         f2: HashSet<Option<i8>>,
@@ -341,7 +341,7 @@ fn inner_nullable() {
     }
 
     #[derive(ForyObject, Debug)]
-    #[fory_debug]
+    #[fory(debug)]
     pub struct Item2 {
         f1: Vec<String>,
         f2: HashSet<i8>,
@@ -371,7 +371,7 @@ fn inner_nullable() {
 #[test]
 fn nullable_struct() {
     #[derive(ForyObject, Debug, PartialEq)]
-    #[fory_debug]
+    #[fory(debug)]
     pub struct Item {
         name: String,
         data: Vec<Option<String>>,
@@ -379,7 +379,7 @@ fn nullable_struct() {
     }
 
     #[derive(ForyObject, Debug)]
-    #[fory_debug]
+    #[fory(debug)]
     pub struct Person1 {
         f1: Item,
         f2: Option<Item>,
@@ -388,7 +388,7 @@ fn nullable_struct() {
     }
 
     #[derive(ForyObject, Debug)]
-    #[fory_debug]
+    #[fory(debug)]
     pub struct Person2 {
         f1: Option<Item>,
         f2: Item,
@@ -443,7 +443,7 @@ fn enum_without_payload() {
         Blue,
     }
     #[derive(ForyObject, Debug, PartialEq)]
-    #[fory_debug]
+    #[fory(debug)]
     struct Person1 {
         f1: Color1,
         f2: Color1,
@@ -456,7 +456,7 @@ fn enum_without_payload() {
         last: i8,
     }
     #[derive(ForyObject, Debug, PartialEq)]
-    #[fory_debug]
+    #[fory(debug)]
     struct Person2 {
         // same
         f1: Color1,
@@ -512,7 +512,7 @@ fn named_enum() {
         White,
     }
     #[derive(ForyObject, Debug, PartialEq)]
-    #[fory_debug]
+    #[fory(debug)]
     struct Item1 {
         f1: Color,
         f2: Color,
@@ -527,7 +527,7 @@ fn named_enum() {
         last: i8,
     }
     #[derive(ForyObject, Debug, PartialEq)]
-    #[fory_debug]
+    #[fory(debug)]
     struct Item2 {
         f1: Color,
         f2: Option<Color>,
